/**** A P P L I C A T I O N   N O T E   ************************************
*
* Title			: 8ch forward phase control dimmer
* Version		: v4.6
* Last updated	: 19.06.2014
* Target		: PC_DIMMER DMX-Dimmerpack [ATtiny2313]
* Clock			: 8-16MHz
* MainsFreq		: 50Hz
*
* Written by Hendrik Hoelscher, http://www.hoelscher-hi.de
* Enhanced by Christian Noeding, http://www.pcdimmer.de
***************************************************************************
 This program is free software; you can redistribute it and/or 
 modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version2 of 
 the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 General Public License for more details. 

 If you have no copy of the GNU General Public License, write to the 
 Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

 For other license models, please contact the author.

;***************************************************************************/
#include "main.h"

// ********************* local definitions *********************

enum {state_IDLE, state_READSTARTBYTE, state_FINDSTARTADDRESS, state_READDATA};		//DMX states

volatile uint8_t 	 gDmxState;


// *************** DMX Reception Initialisation ****************
void init_DMX(void)
{
	DDRD |= (1<<2);
	UBRRH  = 0;
	UBRRL  = ((F_OSC/4000)-1);						//250kbaud, 8N2
	//UCSRC |= (1<<URSEL)|(3<<UCSZ0)|(1<<USBS);
	UCSRC |= (1<<UCSZ1)|(1<<UCSZ0)|(1<<USBS);
	UCSRB |= (0<<UCSZ2)|(1<<RXEN)|(1<<RXCIE);
	gDmxState= state_IDLE;
	DmxAlive=255;
}


// *************** DMX Reception ISR ****************
ISR (USART_RX_vect)
{
	static uint16_t DmxCount;
	static uint16_t CurrentDmxByte;
	
	uint8_t USARTstate= UCSRA;						//get state before data!
	uint8_t DmxByte= UDR;							//get data
	uint8_t DmxState= gDmxState;					//just load once from SRAM to increase speed
	 
	if (USARTstate &(1<<FE))						//check for break
	{
		// BREAK
		UCSRA &= ~(1<<FE);							//reset flag
		DmxCount= DmxAddress;						//reset frame counter
		//#ifdef Doubleclock
		//	gDmxState= state_FINDSTARTADDRESS;		// 16MHz -> goto state_STARTADDRESS (Startbyte kann hier nicht empfangen werden)
		//#else
			gDmxState= state_READSTARTBYTE;			// 8MHz -> goto state_STARTBYTE
		//#endif
	}
	else if (DmxState == state_READSTARTBYTE)		// Break passed -> read Startbyte
	{
		// STARTBYTE
		CurrentDmxByte=0;							// CurrentDmxByte=0 for startbyte
		if (DmxByte == 0) 
		{
			gDmxState= state_FINDSTARTADDRESS;		//normal startcode detected
		}
		else
		{
			gDmxState= state_IDLE;					//startbyte>0 -> do nothing
		}
	}
	else if (DmxState == state_FINDSTARTADDRESS)	// ch1, ch2, ch3, ...
	{
		// Startaddress
		CurrentDmxByte=1;							// CurrentDmxByte=1 for first channel
		if (--DmxCount == 0)						// wait for startaddress by decrementing DmxCount, until 0 
		{
			// STARTADDRESS REACHED -> store Data to array element 0
			#ifdef InvertDMX
				DmxField[0]= 255-DmxByte;
			#else
				DmxField[0]= DmxByte;
			#endif
			DmxCount=1;								//set DmxCount to channel 2
			gDmxState = state_READDATA;
		}
	}
	else if (DmxState == state_READDATA)
	{
		// ALL CHANNELS BETWEEN STARTADDRESS AND ENDADDRESS
		CurrentDmxByte++;
		#ifdef InvertDMX
			DmxField[DmxCount]= 255-DmxByte;
		#else
			DmxField[DmxCount]= DmxByte;
		#endif
		DmxCount++;
		if (DmxCount >= DMX_CHANNELS) 	//all ch received?
		{
			gDmxState= state_IDLE;
		}
	}
	else if (DmxState == state_IDLE)
	{
		// ALL CHANNELS ABOVE THE ENDADDRESS
		CurrentDmxByte++;
	}		

	#ifndef UseHardcodedStartaddress
		if (DmxAddressSetup & (1<<GNDA))
		{
			if (CurrentDmxByte==511 && !(DmxAddressSetup & (1<<C512)))
			{
				if (DmxByte>0)
				{
					DmxAddressSetup|=(1<<NAA255);
				}
				DmxAddressSetup|=(1<<C511);
			}
			if (CurrentDmxByte==512 && (DmxAddressSetup & (1<<C511)))
			{
				if (DmxAddressSetup & (1<<NAA255))
				{
					DmxAddress=DmxByte+256;
				}
				else
				{
					DmxAddress=DmxByte+1;
				}
				DmxAddressSetup&=~(1<<GNDA);
				DmxAddressSetup|=(1<<C512);
			}
		}
	#endif
	
	DmxAlive=255;							
}
