/**** A P P L I C A T I O N   N O T E   ************************************
*
* Title			: 8ch forward phase control dimmer
* Version		: v4.6
* Last updated	: 19.06.2014
* Target		: PC_DIMMER DMX-Dimmerpack [ATtiny2313]
* Clock			: 8-16MHz
* MainsFreq		: 50Hz
*
* Written by Christian Noeding, http://www.pcdimmer.de
***************************************************************************
 This program is free software; you can redistribute it and/or 
 modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version2 of 
 the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 General Public License for more details. 

 If you have no copy of the GNU General Public License, write to the 
 Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

 For other license models, please contact the author.

;***************************************************************************/
#include "main.h"

void init_pcdimmer(void)
{
	DDRD = (0<<TASTER)|(1<<LEDGREEN)|(1<<LEDYELLOW)|(1<<CH9OUT);	// Taster-Pin als Eingang, sowie LED-Pins und AuxPins als Ausgang definieren
	PORTD |= (1<<TASTER);							// Pullups fr Taster einschalten

	ledGreenOff();									// Grne LED ausschalten
	ledYellowOn();									// Gelbe LED einschalten
	_delay_ms(250);
	ledBothToggle();								// Gelbe LED ausschalten, Grne LED einschalten
	_delay_ms(250);

	// DMX-Startwerte
	#ifdef InvertDMX
		DmxField[0]=255;
		DmxField[1]=255;
		DmxField[2]=255;
		DmxField[3]=255;
		DmxField[4]=255;
		DmxField[5]=255;
		DmxField[6]=255;
		DmxField[7]=255;
		#ifdef Use9ch
			DmxField[8]=255;
		#endif
	#else
		DmxField[0]=0;
		DmxField[1]=0;
		DmxField[2]=0;
		DmxField[3]=0;
		DmxField[4]=0;
		DmxField[5]=0;
		DmxField[6]=0;
		DmxField[7]=0;
		#ifdef Use9ch
			DmxField[8]=0;
		#endif
	#endif
}

#ifndef UseHardcodedStartaddress
	void CatchAddressByDmx(void)
	{
		DmxAddress=1;
		DmxAddressSetup=(1<<GNDA);	// DmxSetup auf GetNewDmxAddress einstellen

		// Warten, bis neue Adresse empfangen
		ledBothOff();								// Beide LEDs ausschalten
		while ( !((DmxAddressSetup & (1<<C511)) && (DmxAddressSetup & (C512))) )
		{
			ledGreenToggle();						// Grne LED toggeln
			_delay_ms(50);
		}
					
		eeprom_write_word((uint16_t*) 0x78, DmxAddress);
				
		// Warten, bis User die Taste loslsst
		ledBothOff();								// Beide LEDs ausschalten
		while ( !(PIND & (1<<TASTER)) )
		{
			// Taster noch gedrckt
			ledBothToggle();						// Beide LEDs toggeln
			_delay_ms(50);
		}
	}
#endif

int main(void)
{
	init_pcdimmer();								// PC_DIMMER Hardware initialisieren

	// DMX Adresse festlegen
	#ifdef UseHardcodedStartaddress
		DmxAddress=HardcodedStartaddress;
	#else
		DmxAddress=eeprom_read_word((uint16_t*) 0x78);
		if ( (DmxAddress<1) | (DmxAddress>512) )
		{
			DmxAddress=1;
		}
	#endif

	init_DMX();										// DMX512-Empfang initialisieren

	#ifndef UseHardcodedStartaddress
		if ( !(PIND & (1<<TASTER)) )
		{
			// Taste gedrckt -> Adresse auf 1 resetten und im EEPROM abspeichern
			sei();
			DmxAddress=1;
			eeprom_write_word((uint16_t*) 0x78, DmxAddress);
			cli();
		};
	#endif

	// init_230v_PWM aktiviert auch wieder die Interrupts
	init_PWM();										// PWM und Synchronisation initialisieren


	// Lokale Variablen definieren
	uint8_t	DmxValueChanged=0;
	uint8_t	i=0;
	
	for(;;)
	{
		ledGreenOn();								// Grne LED einschalten
	
		#ifdef HasZeroCrossingDetection
			if (GridAlive==0)						// Netzfehler
			{
				#ifdef UsePLLTuner
					ledBothOn();
					while( GridAlive==0 )			// Solange blinken, bis Timer synchron zu AC-Netz
					{
						ledBothToggle();
						_delay_ms(50);
					}

					// Timer1 aktivieren
					TIMSK |= (1<<OCIE1A);			//enable compare irq
					TIFR   = (1<<OCF1A);			//clear Timer flag
					OCR1AH = myOCR1AH;
					OCR1AL = myOCR1AL;
				#else
					//Timer1 aktivieren
					TCCR1A= 0;
					TCCR1B= (1<<CS11);				//set T1 @clk/8 (Prescaler=8)
	
					MCUCR  = (1<<ISC00);			//irq0 on logical change (somit alle 10ms)
					GIMSK  = (1<<INT0);				//enable INT0 for zc
					EIFR   = (1<<INTF0);			//clear INT0 flag

					ledBothOn();
					while( GridAlive==0 )			// Solange blinken, bis Timer synchron zu AC-Netz
					{
						ledBothToggle();
						_delay_ms(50);
					}
				#endif
			}
		#endif

		if (DmxAlive==0)
		{
			// Fehler in DMX-Signal
			ledYellowToggle();						// Gelbe LED toggeln
			_delay_ms(50);
		}else
		{
			// DMX-Signal OK

			// Prfen, ob ein Kanal verndert wurde
			for (i=0; i<DMX_CHANNELS; i++)
			{
				if (DmxOldField[i]!=DmxField[i]){
					DmxValueChanged=1;
					DmxOldField[i]=DmxField[i];
				}
			}
			// Sofern neue Daten eingegangen sind, dies anzeigen
			if (DmxValueChanged==1){
				DmxValueChanged=0;
				ledYellowOn();
				_delay_ms(10);
			}
			ledYellowOff();							// Gelbe LED ausschalten
			DmxAlive--;

			#ifndef UseHardcodedStartaddress
				// Adressnderung whrend der Laufzeit
				if ( !(PIND & (1<<TASTER)) )
				{
					// Taste gedrckt
					i=0;
					while ( !(PIND & (1<<TASTER)) && (i<10) )
					{
						ledGreenToggle();			// Grne LED toggeln
						i++;
						_delay_ms(150);
					}
					if ( !(PIND & (1<<TASTER)) )
					{
						// Taste weiterhin gedrckt -> neue Adresse holen
						CatchAddressByDmx();
					}
				}
			#endif
		}
	}
}
