// Vorabdefinitionen
//#define Doubleclock								//Definieren, um einen 16MHz Quarz zu verwenden. Andernfalls wird 8MHz verwendet

#ifdef Doubleclock
	#define F_CPU 16000000UL     				//Quarz mit 16.000 Mhz
	#define F_OSC 16000		  					//Quarzfrequenz in kHz
#else
	#define F_CPU 8000000UL     				//Quarz mit 8.000 Mhz
	#define F_OSC 8000		  					//Quarzfrequenz in kHz
#endif

// Compilerschalter fr verschiedene Firmwarefunktionen
///////////////////////////////////////////////////////
#define Is230V_Dimmerpack						//Einbinden, wenn Firmware fr 230V Dimmerpack
//#define Is230V_Switchpack						//Einbinden, wenn Firmware fr 230V Switchpack
//#define IsDemultiplexer						//Einbinden, wenn Firmware fr LED- oder Analogdimmerpacks -> "#define UsePLLTuner" verwenden!

//#define UsePLLTuner							//Einbinden, um PLL Tuner statt H.Hlscher ZCD zu verwenden
//#define SixtyHertz							//Einbinden, fr 60Hz Netzfrequenz

#define HasZeroCrossingDetection				//Einbinden, wenn Hardware ber ZC-Detection verfgt
//#define InvertDMX								//Einbinden, wenn die Ausgabe invertiert sein soll (DMX-Eingangswerte werden invertiert)
//#define Use9ch								//Einbinden, um 9. Ausgangskanal (CH9OUT) an PD3 zu verwenden
//#define InvertPhase							//Einbinen, wenn Photodiode des Optokopplers gegen GND und nicht gegen VCC geschaltet ist.

//#define UseHardcodedStartaddress				//Einbinden, um feste Startadresse zu nutzen
//#define HardcodedStartaddress 1				//Einbinden, um fest einprogrammierte DMX-Startadresse zu definieren

//#define IsHardwareRevision10					//Einbinden, um alte Hardware zu verwenden (Taster=PD6)
///////////////////////////////////////////////////////
// GPIO-Konfiguration
#define	LEDGREEN	PD4
#define LEDYELLOW	PD5

#ifdef IsHardwareRevision10
	#define TASTER		PD6 // Older Boards (v1.0/v1.1) Taster auf PD6
	#define CH9OUT		PD3 // Older Boards (v1.0/v1.1) PD3 ist allerdings nicht auf Pinleiste erreichbar
#else
	#define TASTER		PD3 // Boards >= v1.2 PD3
	#define CH9OUT		PD6 // Boards >= v1.2 PD6
#endif
///////////////////////////////////////////////////////

// Definition der Kanalanzahl
#ifdef Use9ch
	#define DMX_CHANNELS 9						//verwende 9 Kanle
#else
	#define DMX_CHANNELS 8						//verwende nur 8 Kanle
#endif

// Standard-Includings 
#include <ctype.h>
//#include <stdbool.h>
#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

// Variablendefinitionen
volatile uint8_t	GridTicks;					//Zhler fr Frequenzmessung
volatile uint8_t	GridAlive;					//Boolscher Wert fr ZCD-Lost-Erkennung

volatile uint8_t	DmxField[DMX_CHANNELS];		//DMX-Werte-Array (0..255)
uint8_t				DmxOldField[DMX_CHANNELS];	//DMX-Werte-Array (derzeit nur um Vernderungen zu erkennen und LED blinken zu lassen)
volatile uint16_t	DmxAddress;					//Startadresse
volatile uint8_t	DmxAlive;					//Zhler fr DMX-Lost-Erkennung

#ifndef UseHardcodedStartaddress
	// Um RAM zu sparen, werden einzelne Bits des Bytes fr die Einstellung verwendet
	volatile uint8_t DmxAddressSetup;		//Bit 0: GetNewDmxAddress, Bit 1: Catched511, Bit 2: Catched512, Bit 3: NewAddressAbove255
	#define	GNDA	0	// GetNewDMXAddress
	#define	C511	1	// Catched511
	#define	C512	2	// Catched512
	#define	NAA255	4	// NewAddressAbove255
#endif

// Berechnung der Timer1 Geschwindigkeit. Pro Halbwelle sollen 
#ifdef SixtyHertz
	#define MinimumSawtooth 10					// Sgezahn luft von 255 herunter bis MinimumSawtooth=10 (255..10=246 Steps)

	#ifdef Doubleclock
		#define myOCR1AH 0x02;					// Sinusperiodenzeit=(1/16Mhz)*246*320*2
		#define myOCR1AL 0x1E;					// OCR1A=(16MHz/(246*2*60Hz))-1=542=0x021E
	#else
		#define myOCR1AH 0x01;					// Sinusperiodenzeit=(1/8Mhz)*246*320*2
		#define myOCR1AL 0x0F;					// OCR1A=(8MHz/(246*2*60Hz))-1=271=0x010F
	#endif
#else
	#define MinimumSawtooth 6					// Sgezahn luft von 255 herunter bis MinimumSawtooth=6 (255..6=250 Steps)

	#ifdef Doubleclock
		#define myOCR1AH 0x02;					// Sinusperiodenzeit=(1/16Mhz)*250*320*2
		#define myOCR1AL 0x7F;					// OCR1A=(16MHz/(250*2*50Hz))-1=639=0x027F
	#else
		#define myOCR1AH 0x01;					// Sinusperiodenzeit=(1/8Mhz)*250*320*2
		#define myOCR1AL 0x3F;					// OCR1A=(8MHz/(250*2*50Hz))-1=319=0x013F
	#endif
#endif

// Sicherheitsabstand zur nchsten Halbwelle, um Fehlzndungen der TRIACs zu vermeiden. x*(10ms/250)
#define GridSaftyMargin 35					// Bei 35 -> 35*(10ms/250) = 1,4ms

// Beispiel fr 16MHz und 50Hz:
// Timer0 gibt Ticks aus. Ein Tick entspricht 1024/16000=0,064ms. Da zwischen zwei INT 10ms vergehen: 10ms/0,064ms = 156 Ticks zwischen Interrupts
#ifdef SixtyHertz
	#ifdef Doubleclock
		// 2,56Hz Toleranzgrenze
		#define TopGridTicks 150				// ((1000*0,5)/60)/(1024/16000) = 8,33ms/0,064ms = 130+20
		#define BotGridTicks 110				// ((1000*0,5)/60)/(1024/16000) = 8,33ms/0,064ms = 130-20
	#else
		// 2,56Hz Toleranzgrenze
		#define TopGridTicks 75 				// ((1000*0,5)/60)/(1024/8000) = 8,33ms/0,128ms = 65+10
		#define BotGridTicks 55 				// ((1000*0,5)/60)/(1024/8000) = 8,33ms/0,128ms = 65-10
	#endif
#else
	#ifdef Doubleclock
		// 2,56Hz Toleranzgrenze
		#define TopGridTicks 176 				// ((1000*0,5)/50)/(1024/16000) = 10ms/0,064ms = 156+20
		#define BotGridTicks 136				// ((1000*0,5)/50)/(1024/16000) = 10ms/0,064ms = 156-20
	#else
		// 2,56Hz Toleranzgrenze
		#define TopGridTicks 88 				// ((1000*0,5)/50)/(1024/8000) = 10ms/0,128ms = 78+10
		#define BotGridTicks 68 				// ((1000*0,5)/50)/(1024/8000) = 10ms/0,128ms = 78-10
	#endif
#endif

#define TRUE 1
#define FALSE 0

// Funktionsprototypen
extern void init_DMX(void);
extern void init_PWM(void);

// GPIO Funktionen definieren
#define ledYellowOn()    	PORTD |= (1<<LEDYELLOW)
#define ledYellowOff()   	PORTD &=~(1<<LEDYELLOW)
#define ledYellowToggle()	PORTD ^= (1<<LEDYELLOW)
#define ledGreenOn()  		PORTD |= (1<<LEDGREEN)
#define ledGreenOff() 		PORTD &=~(1<<LEDGREEN)
#define ledGreenToggle()	PORTD ^= (1<<LEDGREEN)
#define ledBothOn()			PORTD |= (1<<LEDYELLOW)|(1<<LEDGREEN)
#define ledBothOff()		PORTD &=~(1<<LEDYELLOW)|~(1<<LEDGREEN)
#define ledBothToggle()		PORTD ^= (1<<LEDYELLOW)|(1<<LEDGREEN)

// Plausibilittstest der Compilerschalter
#ifdef Is230V_Dimmerpack
	#ifndef HasZeroCrossingDetection
		#define HasZeroCrossingDetection
		#warning ZeroCrossing-Detection wurde automatisch aktiviert
	#endif
#endif
#ifdef Is230V_Switchpack
	#ifndef UsePLLTuner
		#define UsePLLTuner
		#warning PLL-Tuner wurde automatisch aktiviert
	#endif
#endif
#ifdef IsDemultiplexer
	#ifndef UsePLLTuner
		#define UsePLLTuner
		#warning PLL-Tuner wurde automatisch aktiviert
	#endif
	#ifdef HasZeroCrossingDetection
		#error "HasZeroCrossingDetection" does not make sense - remove Definition in HeaderFile!
	#endif
#endif
